function [find,vaf,targone,targtwo,outpermone,outpermtwo] = ...
    bicirsarobfnd(prox,inperm,kblock)

% BICIRSAROBFND fits the sum of two strongly circular-anti-Robinson
% matrices using iterative projection to a symmetric proximity
% matrix in the $L_{2}$-norm based on permutations 
% identified through the use of iterative quadratic assignment.
%
% syntax: [find,vaf,targone,targtwo,outpermone,outpermtwo] = ...
%      bicirsarobfnd(prox,inperm,kblock)
%
% PROX is the input proximity matrix ($n \times n$ with a zero main 
% diagonal and a dissimilarity interpretation);
% INPERM is a given starting permutation of the first $n$ integers;
% FIND is the least-squares optimal matrix (with variance-
% accounted-for of VAF) to PROX and is the 
% sum of the two strongly circular-anti-Robinson matrices;
% TARGONE and TARGTWO are based on the two row and column
% object orderings given by the ending permutations OUTPERMONE 
% and OUTPERMTWO. KBLOCK defines the block size in the use the
% iterative quadratic assignment routine.

n = size(prox,1);
proxave = zeros(n,n);
resprox = zeros(n,n);

[targ1,vaftarg1,outperm1] = cirsarobfnd_ac(prox,inperm,kblock);

resprox1(outperm1,outperm1) = prox(outperm1,outperm1) - targ1;

[targ2,vaftarg2,outperm2] = cirsarobfnd_ac(resprox1,inperm,kblock);

resprox2(outperm2,outperm2) = resprox1(outperm2,outperm2) - targ2;

find = prox - resprox2;

aveprox = sum(sum(prox))/(n*(n-1));

for i = 1:n
    for j = 1:n
        if(i ~= j)
            proxave(i,j) = aveprox;
        else
            proxave(i,j) = 0.0;
        end
    end
end

diff = sum(sum((prox - find).^2));
denom = sum(sum((prox - proxave).^2));
vaf = 1 - (diff/denom);
            
targone = targ1;
targtwo = targ2;
outpermone = outperm1;
outpermtwo = outperm2;

vafdiff = 1.0;
iteration = 0;

while ((vafdiff >= 1.0e-006) && (iteration <= 100))
    
    vafprev = vaf;
    iteration = iteration + 1;
   
    resprox(outpermtwo,outpermtwo) = prox(outpermtwo,outpermtwo) - targtwo;
    [targone,vafone] = cirsarobfit(resprox,outpermone,targone);
    
    resprox(outpermone,outpermone) = prox(outpermone,outpermone) - targone;
    [targtwo,vaftwo] = cirsarobfit(resprox,outpermtwo,targtwo);
    
    find(outpermone,outpermone) = targone;
    find(outpermtwo,outpermtwo) = find(outpermtwo,outpermtwo) + targtwo;
    
    diff = sum(sum((prox - find).^2));
    denom = sum(sum((prox - proxave).^2));
    vaf = 1 - (diff/denom);
    
    vafdiff = abs(vaf - vafprev);
end